/* This file was generated by syncqt. */
#ifndef QT_QTLABSSYNCHRONIZER_VERSION_H
#define QT_QTLABSSYNCHRONIZER_VERSION_H

#define QTLABSSYNCHRONIZER_VERSION_STR "6.11.0"

#define QTLABSSYNCHRONIZER_VERSION 0x0601100

#endif // QT_QTLABSSYNCHRONIZER_VERSION_H
